/*********************************************************************
*                     SEGGER Microcontroller GmbH                    *
*                        The Embedded Experts                        *
**********************************************************************
*                                                                    *
*       (c) 1995 - 2023 SEGGER Microcontroller GmbH                  *
*                                                                    *
*       Internet: segger.com  Support: support_embos@segger.com      *
*                                                                    *
**********************************************************************
*                                                                    *
*       embOS-Ultra * Real time operating system                     *
*                                                                    *
*       Please note:                                                 *
*                                                                    *
*       Knowledge of this file may under no circumstances            *
*       be used to write a similar product or a real-time            *
*       operating system for in-house use.                           *
*                                                                    *
*       Thank you for your fairness !                                *
*                                                                    *
**********************************************************************
*                                                                    *
*       OS version: V5.18.0.0                                        *
*                                                                    *
**********************************************************************

-------------------------- END-OF-HEADER -----------------------------

File      : XMC4800_Vectors.s
Purpose   : Exception and interrupt vectors for XMC4800 devices.

Additional information:
  Preprocessor Definitions
    __NO_EXTERNAL_INTERRUPTS
      If defined,
        the vector table will contain only the internal exceptions
        and interrupts.
    __VECTORS_IN_RAM
      If defined,
        an area of RAM, large enough to store the vector table,
        will be reserved.

    __OPTIMIZATION_SMALL
      If defined,
        all weak definitions of interrupt handlers will share the
        same implementation.
      If not defined,
        all weak definitions of interrupt handlers will be defined
        with their own implementation.
*/
        .syntax unified

/*********************************************************************
*
*       Macros
*
**********************************************************************
*/

//
// Directly place a vector (word) in the vector table
//
.macro VECTOR Name=
        .section .vectors, "ax"
        .code 16
        .word \Name
.endm

//
// Declare an exception handler with a weak definition
//
.macro EXC_HANDLER Name=
        //
        // Insert vector in vector table
        //
        .section .vectors, "ax"
        .word \Name
        //
        // Insert dummy handler in init section
        //
        .section .init.\Name, "ax"
        .thumb_func
        .weak \Name
        .balign 2
\Name:
        1: b 1b   // Endless loop
.endm

//
// Declare an interrupt handler with a weak definition
//
.macro ISR_HANDLER Name=
        //
        // Insert vector in vector table
        //
        .section .vectors, "ax"
        .word \Name
        //
        // Insert dummy handler in init section
        //
#if defined(__OPTIMIZATION_SMALL)
        .section .init, "ax"
        .weak \Name
        .thumb_set \Name,Dummy_Handler
#else
        .section .init.\Name, "ax"
        .thumb_func
        .weak \Name
        .balign 2
\Name:
        1: b 1b   // Endless loop
#endif
.endm

//
// Place a reserved vector in vector table
//
.macro ISR_RESERVED
        .section .vectors, "ax"
        .word 0
.endm

//
// Place a reserved vector in vector table
//
.macro ISR_RESERVED_DUMMY
        .section .vectors, "ax"
        .word Dummy_Handler
.endm

/*********************************************************************
*
*       Externals
*
**********************************************************************
*/
        .extern __stack_end__
        .extern Reset_Handler
        .extern HardFault_Handler

/*********************************************************************
*
*       Global functions
*
**********************************************************************
*/

/*********************************************************************
*
*  Setup of the vector table and weak definition of interrupt handlers
*
*/
        .section .vectors, "ax"
        .code 16
        .balign 512
        .global _vectors
        .global __Vectors
_vectors:
__Vectors:
        //
        // Internal exceptions and interrupts
        //
        VECTOR __stack_end__
        VECTOR Reset_Handler
        EXC_HANDLER NMI_Handler
        VECTOR HardFault_Handler
        ISR_RESERVED
        ISR_RESERVED
        ISR_RESERVED
        ISR_RESERVED
        ISR_RESERVED
        ISR_RESERVED
        ISR_RESERVED
        EXC_HANDLER SVC_Handler
        ISR_RESERVED
        ISR_RESERVED
        EXC_HANDLER PendSV_Handler
        EXC_HANDLER SysTick_Handler
        //
        // External interrupts
        //
#ifndef __NO_EXTERNAL_INTERRUPTS
        ISR_HANDLER SCU_0_IRQHandler
        ISR_HANDLER ERU0_0_IRQHandler
        ISR_HANDLER ERU0_1_IRQHandler
        ISR_HANDLER ERU0_2_IRQHandler
        ISR_HANDLER ERU0_3_IRQHandler
        ISR_HANDLER ERU1_0_IRQHandler
        ISR_HANDLER ERU1_1_IRQHandler
        ISR_HANDLER ERU1_2_IRQHandler
        ISR_HANDLER ERU1_3_IRQHandler
        ISR_RESERVED_DUMMY
        ISR_RESERVED_DUMMY
        ISR_RESERVED_DUMMY
        ISR_HANDLER PMU0_0_IRQHandler
        ISR_RESERVED_DUMMY
        ISR_HANDLER VADC0_C0_0_IRQHandler
        ISR_HANDLER VADC0_C0_1_IRQHandler
        ISR_HANDLER VADC0_C0_2_IRQHandler
        ISR_HANDLER VADC0_C0_3_IRQHandler
        ISR_HANDLER VADC0_G0_0_IRQHandler
        ISR_HANDLER VADC0_G0_1_IRQHandler
        ISR_HANDLER VADC0_G0_2_IRQHandler
        ISR_HANDLER VADC0_G0_3_IRQHandler
        ISR_HANDLER VADC0_G1_0_IRQHandler
        ISR_HANDLER VADC0_G1_1_IRQHandler
        ISR_HANDLER VADC0_G1_2_IRQHandler
        ISR_HANDLER VADC0_G1_3_IRQHandler
        ISR_HANDLER VADC0_G2_0_IRQHandler
        ISR_HANDLER VADC0_G2_1_IRQHandler
        ISR_HANDLER VADC0_G2_2_IRQHandler
        ISR_HANDLER VADC0_G2_3_IRQHandler
        ISR_HANDLER VADC0_G3_0_IRQHandler
        ISR_HANDLER VADC0_G3_1_IRQHandler
        ISR_HANDLER VADC0_G3_2_IRQHandler
        ISR_HANDLER VADC0_G3_3_IRQHandler
        ISR_HANDLER DSD0_0_IRQHandler
        ISR_HANDLER DSD0_1_IRQHandler
        ISR_HANDLER DSD0_2_IRQHandler
        ISR_HANDLER DSD0_3_IRQHandler
        ISR_HANDLER DSD0_4_IRQHandler
        ISR_HANDLER DSD0_5_IRQHandler
        ISR_HANDLER DSD0_6_IRQHandler
        ISR_HANDLER DSD0_7_IRQHandler
        ISR_HANDLER DAC0_0_IRQHandler
        ISR_HANDLER DAC0_1_IRQHandler
        ISR_HANDLER CCU40_0_IRQHandler
        ISR_HANDLER CCU40_1_IRQHandler
        ISR_HANDLER CCU40_2_IRQHandler
        ISR_HANDLER CCU40_3_IRQHandler
        ISR_HANDLER CCU41_0_IRQHandler
        ISR_HANDLER CCU41_1_IRQHandler
        ISR_HANDLER CCU41_2_IRQHandler
        ISR_HANDLER CCU41_3_IRQHandler
        ISR_HANDLER CCU42_0_IRQHandler
        ISR_HANDLER CCU42_1_IRQHandler
        ISR_HANDLER CCU42_2_IRQHandler
        ISR_HANDLER CCU42_3_IRQHandler
        ISR_HANDLER CCU43_0_IRQHandler
        ISR_HANDLER CCU43_1_IRQHandler
        ISR_HANDLER CCU43_2_IRQHandler
        ISR_HANDLER CCU43_3_IRQHandler
        ISR_HANDLER CCU80_0_IRQHandler
        ISR_HANDLER CCU80_1_IRQHandler
        ISR_HANDLER CCU80_2_IRQHandler
        ISR_HANDLER CCU80_3_IRQHandler
        ISR_HANDLER CCU81_0_IRQHandler
        ISR_HANDLER CCU81_1_IRQHandler
        ISR_HANDLER CCU81_2_IRQHandler
        ISR_HANDLER CCU81_3_IRQHandler
        ISR_HANDLER POSIF0_0_IRQHandler
        ISR_HANDLER POSIF0_1_IRQHandler
        ISR_HANDLER POSIF1_0_IRQHandler
        ISR_HANDLER POSIF1_1_IRQHandler
        ISR_RESERVED_DUMMY
        ISR_RESERVED_DUMMY
        ISR_RESERVED_DUMMY
        ISR_RESERVED_DUMMY
        ISR_HANDLER CAN0_0_IRQHandler
        ISR_HANDLER CAN0_1_IRQHandler
        ISR_HANDLER CAN0_2_IRQHandler
        ISR_HANDLER CAN0_3_IRQHandler
        ISR_HANDLER CAN0_4_IRQHandler
        ISR_HANDLER CAN0_5_IRQHandler
        ISR_HANDLER CAN0_6_IRQHandler
        ISR_HANDLER CAN0_7_IRQHandler
        ISR_HANDLER USIC0_0_IRQHandler
        ISR_HANDLER USIC0_1_IRQHandler
        ISR_HANDLER USIC0_2_IRQHandler
        ISR_HANDLER USIC0_3_IRQHandler
        ISR_HANDLER USIC0_4_IRQHandler
        ISR_HANDLER USIC0_5_IRQHandler
        ISR_HANDLER USIC1_0_IRQHandler
        ISR_HANDLER USIC1_1_IRQHandler
        ISR_HANDLER USIC1_2_IRQHandler
        ISR_HANDLER USIC1_3_IRQHandler
        ISR_HANDLER USIC1_4_IRQHandler
        ISR_HANDLER USIC1_5_IRQHandler
        ISR_HANDLER USIC2_0_IRQHandler
        ISR_HANDLER USIC2_1_IRQHandler
        ISR_HANDLER USIC2_2_IRQHandler
        ISR_HANDLER USIC2_3_IRQHandler
        ISR_HANDLER USIC2_4_IRQHandler
        ISR_HANDLER USIC2_5_IRQHandler
        ISR_HANDLER LEDTS0_0_IRQHandler
        ISR_RESERVED_DUMMY
        ISR_HANDLER FCE0_0_IRQHandler
        ISR_HANDLER GPDMA0_0_IRQHandler
        ISR_HANDLER SDMMC0_0_IRQHandler
        ISR_HANDLER USB0_0_IRQHandler
        ISR_HANDLER ETH0_0_IRQHandler
        ISR_HANDLER ECAT0_0_IRQHandler
        ISR_HANDLER GPDMA1_0_IRQHandler
#endif
        //
        .section .vectors, "ax"
_vectors_end:

#ifdef __VECTORS_IN_RAM
        //
        // Reserve space with the size of the vector table
        // in the designated RAM section.
        //
        .section .vectors_ram, "ax"
        .balign 512
        .global _vectors_ram

_vectors_ram:
        .space _vectors_end - _vectors, 0
#endif

/*********************************************************************
*
*  Dummy handler to be used for reserved interrupt vectors
*  and weak implementation of interrupts.
*
*/
        .section .init.Dummy_Handler, "ax"
        .thumb_func
        .weak Dummy_Handler
        .balign 2
Dummy_Handler:
        1: b 1b   // Endless loop


/*************************** End of file ****************************/
