/*********************************************************************
*                     SEGGER Microcontroller GmbH                    *
*                        The Embedded Experts                        *
**********************************************************************
*                                                                    *
*       (c) 1995 - 2023 SEGGER Microcontroller GmbH                  *
*                                                                    *
*       Internet: segger.com  Support: support_embos@segger.com      *
*                                                                    *
**********************************************************************
*                                                                    *
*       embOS-Ultra * Real time operating system                     *
*                                                                    *
*       Please note:                                                 *
*                                                                    *
*       Knowledge of this file may under no circumstances            *
*       be used to write a similar product or a real-time            *
*       operating system for in-house use.                           *
*                                                                    *
*       Thank you for your fairness !                                *
*                                                                    *
**********************************************************************
*                                                                    *
*       OS version: V5.18.0.0                                        *
*                                                                    *
**********************************************************************

-------------------------- END-OF-HEADER -----------------------------

File      : saml11e16a_Vectors.s
Purpose   : Exception and interrupt vectors for saml11e16a devices.

Additional information:
  Preprocessor Definitions
    __NO_EXTERNAL_INTERRUPTS
      If defined,
        the vector table will contain only the internal exceptions
        and interrupts.
    __VECTORS_IN_RAM
      If defined,
        an area of RAM, large enough to store the vector table,
        will be reserved.

    __OPTIMIZATION_SMALL
      If defined,
        all weak definitions of interrupt handlers will share the
        same implementation.
      If not defined,
        all weak definitions of interrupt handlers will be defined
        with their own implementation.
*/
        .syntax unified

/*********************************************************************
*
*       Macros
*
**********************************************************************
*/

//
// Directly place a vector (word) in the vector table
//
.macro VECTOR Name=
        .section .vectors, "ax"
        .code 16
        .word \Name
.endm

//
// Declare an exception handler with a weak definition
//
.macro EXC_HANDLER Name=
        //
        // Insert vector in vector table
        //
        .section .vectors, "ax"
        .word \Name
        //
        // Insert dummy handler in init section
        //
        .section .init.\Name, "ax"
        .thumb_func
        .weak \Name
        .balign 2
\Name:
        1: b 1b   // Endless loop
.endm

//
// Declare an interrupt handler with a weak definition
//
.macro ISR_HANDLER Name=
        //
        // Insert vector in vector table
        //
        .section .vectors, "ax"
        .word \Name
        //
        // Insert dummy handler in init section
        //
#if defined(__OPTIMIZATION_SMALL)
        .section .init, "ax"
        .weak \Name
        .thumb_set \Name,Dummy_Handler
#else
        .section .init.\Name, "ax"
        .thumb_func
        .weak \Name
        .balign 2
\Name:
        1: b 1b   // Endless loop
#endif
.endm

//
// Place a reserved vector in vector table
//
.macro ISR_RESERVED
        .section .vectors, "ax"
        .word 0
.endm

//
// Place a reserved vector in vector table
//
.macro ISR_RESERVED_DUMMY
        .section .vectors, "ax"
        .word Dummy_Handler
.endm

/*********************************************************************
*
*       Externals
*
**********************************************************************
*/
        .extern __stack_end__
        .extern Reset_Handler
        .extern HardFault_Handler

/*********************************************************************
*
*       Global functions
*
**********************************************************************
*/

/*********************************************************************
*
*  Setup of the vector table and weak definition of interrupt handlers
*
*/
        .section .vectors, "ax"
        .code 16
        .balign 256
        .global _vectors
_vectors:
        //
        // Internal exceptions and interrupts
        //
        VECTOR __stack_end__
        VECTOR Reset_Handler
        EXC_HANDLER NMI_Handler
        VECTOR HardFault_Handler
        ISR_RESERVED
        ISR_RESERVED
        ISR_RESERVED
        ISR_RESERVED
        ISR_RESERVED
        ISR_RESERVED
        ISR_RESERVED
        EXC_HANDLER SVC_Handler
        ISR_RESERVED
        ISR_RESERVED
        EXC_HANDLER PendSV_Handler
        EXC_HANDLER SysTick_Handler
        //
        // External interrupts
        //
#ifndef __NO_EXTERNAL_INTERRUPTS
        ISR_HANDLER SYSTEM_Handler
        ISR_HANDLER WDT_Handler
        ISR_HANDLER RTC_Handler
        ISR_HANDLER EIC_EXTINT_0_Handler
        ISR_HANDLER EIC_EXTINT_1_Handler
        ISR_HANDLER EIC_EXTINT_2_Handler
        ISR_HANDLER EIC_EXTINT_3_Handler
        ISR_HANDLER EIC_OTHER_Handler
        ISR_HANDLER FREQM_Handler
        ISR_HANDLER NVMCTRL_Handler
        ISR_HANDLER PORT_Handler
        ISR_HANDLER DMAC_0_Handler
        ISR_HANDLER DMAC_1_Handler
        ISR_HANDLER DMAC_2_Handler
        ISR_HANDLER DMAC_3_Handler
        ISR_HANDLER DMAC_OTHER_Handler
        ISR_HANDLER EVSYS_0_Handler
        ISR_HANDLER EVSYS_1_Handler
        ISR_HANDLER EVSYS_2_Handler
        ISR_HANDLER EVSYS_3_Handler
        ISR_HANDLER EVSYS_NSCHK_Handler
        ISR_HANDLER PAC_Handler
        ISR_HANDLER SERCOM0_0_Handler
        ISR_HANDLER SERCOM0_1_Handler
        ISR_HANDLER SERCOM0_2_Handler
        ISR_HANDLER SERCOM0_OTHER_Handler
        ISR_HANDLER SERCOM1_0_Handler
        ISR_HANDLER SERCOM1_1_Handler
        ISR_HANDLER SERCOM1_2_Handler
        ISR_HANDLER SERCOM1_OTHER_Handler
        ISR_HANDLER SERCOM2_0_Handler
        ISR_HANDLER SERCOM2_1_Handler
        ISR_HANDLER SERCOM2_2_Handler
        ISR_HANDLER SERCOM2_OTHER_Handler
        ISR_HANDLER TC0_Handler
        ISR_HANDLER TC1_Handler
        ISR_HANDLER TC2_Handler
        ISR_HANDLER ADC_OTHER_Handler
        ISR_HANDLER ADC_RESRDY_Handler
        ISR_HANDLER AC_Handler
        ISR_HANDLER DAC_UNDERRUN_A_Handler
        ISR_HANDLER DAC_EMPTY_Handler
        ISR_HANDLER PTC_Handler
        ISR_HANDLER TRNG_Handler
        ISR_HANDLER TRAM_Handler
#endif
        //
        .section .vectors, "ax"
_vectors_end:

#ifdef __VECTORS_IN_RAM
        //
        // Reserve space with the size of the vector table
        // in the designated RAM section.
        //
        .section .vectors_ram, "ax"
        .balign 256
        .global _vectors_ram

_vectors_ram:
        .space _vectors_end - _vectors, 0
#endif

/*********************************************************************
*
*  Dummy handler to be used for reserved interrupt vectors
*  and weak implementation of interrupts.
*
*/
        .section .init.Dummy_Handler, "ax"
        .thumb_func
        .weak Dummy_Handler
        .balign 2
Dummy_Handler:
        1: b 1b   // Endless loop


/*************************** End of file ****************************/
