/*********************************************************************
*                     SEGGER Microcontroller GmbH                    *
*                        The Embedded Experts                        *
**********************************************************************
*                                                                    *
*       (c) 1995 - 2023 SEGGER Microcontroller GmbH                  *
*                                                                    *
*       Internet: segger.com  Support: support_embos@segger.com      *
*                                                                    *
**********************************************************************
*                                                                    *
*       embOS-Ultra * Real time operating system                     *
*                                                                    *
*       Please note:                                                 *
*                                                                    *
*       Knowledge of this file may under no circumstances            *
*       be used to write a similar product or a real-time            *
*       operating system for in-house use.                           *
*                                                                    *
*       Thank you for your fairness !                                *
*                                                                    *
**********************************************************************
*                                                                    *
*       OS version: V5.18.0.0                                        *
*                                                                    *
**********************************************************************

-------------------------- END-OF-HEADER -----------------------------

File      : samv71q21b_Vectors.s
Purpose   : Exception and interrupt vectors for samv71q21b devices.

Additional information:
  Preprocessor Definitions
    __NO_EXTERNAL_INTERRUPTS
      If defined,
        the vector table will contain only the internal exceptions
        and interrupts.
    __VECTORS_IN_RAM
      If defined,
        an area of RAM, large enough to store the vector table,
        will be reserved.

    __OPTIMIZATION_SMALL
      If defined,
        all weak definitions of interrupt handlers will share the
        same implementation.
      If not defined,
        all weak definitions of interrupt handlers will be defined
        with their own implementation.
*/
        .syntax unified

/*********************************************************************
*
*       Macros
*
**********************************************************************
*/

//
// Directly place a vector (word) in the vector table
//
.macro VECTOR Name=
        .section .vectors, "ax"
        .code 16
        .word \Name
.endm

//
// Declare an exception handler with a weak definition
//
.macro EXC_HANDLER Name=
        //
        // Insert vector in vector table
        //
        .section .vectors, "ax"
        .word \Name
        //
        // Insert dummy handler in init section
        //
        .section .init.\Name, "ax"
        .thumb_func
        .weak \Name
        .balign 2
\Name:
        1: b 1b   // Endless loop
.endm

//
// Declare an interrupt handler with a weak definition
//
.macro ISR_HANDLER Name=
        //
        // Insert vector in vector table
        //
        .section .vectors, "ax"
        .word \Name
        //
        // Insert dummy handler in init section
        //
#if defined(__OPTIMIZATION_SMALL)
        .section .init, "ax"
        .weak \Name
        .thumb_set \Name,Dummy_Handler
#else
        .section .init.\Name, "ax"
        .thumb_func
        .weak \Name
        .balign 2
\Name:
        1: b 1b   // Endless loop
#endif
.endm

//
// Place a reserved vector in vector table
//
.macro ISR_RESERVED
        .section .vectors, "ax"
        .word 0
.endm

//
// Place a reserved vector in vector table
//
.macro ISR_RESERVED_DUMMY
        .section .vectors, "ax"
        .word Dummy_Handler
.endm

/*********************************************************************
*
*       Externals
*
**********************************************************************
*/
        .extern __stack_end__
        .extern Reset_Handler
        .extern HardFault_Handler

/*********************************************************************
*
*       Global functions
*
**********************************************************************
*/

/*********************************************************************
*
*  Setup of the vector table and weak definition of interrupt handlers
*
*/
        .section .vectors, "ax"
        .code 16
        .balign 512
        .global _vectors
_vectors:
        //
        // Internal exceptions and interrupts
        //
        VECTOR __stack_end__
        VECTOR Reset_Handler
        EXC_HANDLER NMI_Handler
        VECTOR HardFault_Handler
        ISR_RESERVED
        ISR_RESERVED
        ISR_RESERVED
        ISR_RESERVED
        ISR_RESERVED
        ISR_RESERVED
        ISR_RESERVED
        EXC_HANDLER SVC_Handler
        ISR_RESERVED
        ISR_RESERVED
        EXC_HANDLER PendSV_Handler
        EXC_HANDLER SysTick_Handler
        //
        // External interrupts
        //
#ifndef __NO_EXTERNAL_INTERRUPTS
        ISR_HANDLER SUPC_Handler
        ISR_HANDLER RSTC_Handler
        ISR_HANDLER RTC_Handler
        ISR_HANDLER RTT_Handler
        ISR_HANDLER WDT_Handler
        ISR_HANDLER PMC_Handler
        ISR_HANDLER EFC_Handler
        ISR_HANDLER UART0_Handler
        ISR_HANDLER UART1_Handler
        ISR_RESERVED_DUMMY
        ISR_HANDLER PIOA_Handler
        ISR_HANDLER PIOB_Handler
        ISR_HANDLER PIOC_Handler
        ISR_HANDLER USART0_Handler
        ISR_HANDLER USART1_Handler
        ISR_HANDLER USART2_Handler
        ISR_HANDLER PIOD_Handler
        ISR_HANDLER PIOE_Handler
        ISR_HANDLER HSMCI_Handler
        ISR_HANDLER TWIHS0_Handler
        ISR_HANDLER TWIHS1_Handler
        ISR_HANDLER SPI0_Handler
        ISR_HANDLER SSC_Handler
        ISR_HANDLER TC0_CH0_Handler
        ISR_HANDLER TC0_CH1_Handler
        ISR_HANDLER TC0_CH2_Handler
        ISR_HANDLER TC1_CH0_Handler
        ISR_HANDLER TC1_CH1_Handler
        ISR_HANDLER TC1_CH2_Handler
        ISR_HANDLER AFEC0_Handler
        ISR_HANDLER DACC_Handler
        ISR_HANDLER PWM0_Handler
        ISR_HANDLER ICM_Handler
        ISR_HANDLER ACC_Handler
        ISR_HANDLER USBHS_Handler
        ISR_HANDLER MCAN0_INT0_Handler
        ISR_HANDLER MCAN0_INT1_Handler
        ISR_HANDLER MCAN1_INT0_Handler
        ISR_HANDLER MCAN1_INT1_Handler
        ISR_HANDLER GMAC_Handler
        ISR_HANDLER AFEC1_Handler
        ISR_HANDLER TWIHS2_Handler
        ISR_HANDLER SPI1_Handler
        ISR_HANDLER QSPI_Handler
        ISR_HANDLER UART2_Handler
        ISR_HANDLER UART3_Handler
        ISR_HANDLER UART4_Handler
        ISR_HANDLER TC2_CH0_Handler
        ISR_HANDLER TC2_CH1_Handler
        ISR_HANDLER TC2_CH2_Handler
        ISR_HANDLER TC3_CH0_Handler
        ISR_HANDLER TC3_CH1_Handler
        ISR_HANDLER TC3_CH2_Handler
        ISR_HANDLER MLB_Handler
        ISR_RESERVED_DUMMY
        ISR_RESERVED_DUMMY
        ISR_HANDLER AES_Handler
        ISR_HANDLER TRNG_Handler
        ISR_HANDLER XDMAC_Handler
        ISR_HANDLER ISI_Handler
        ISR_HANDLER PWM1_Handler
        ISR_HANDLER FPU_Handler
        ISR_HANDLER SDRAMC_Handler
        ISR_HANDLER RSWDT_Handler
        ISR_HANDLER CCW_Handler
        ISR_HANDLER CCF_Handler
        ISR_HANDLER GMAC_Q1_Handler
        ISR_HANDLER GMAC_Q2_Handler
        ISR_HANDLER IXC_Handler
        ISR_HANDLER I2SC0_Handler
        ISR_HANDLER I2SC1_Handler
        ISR_HANDLER GMAC_Q3_Handler
        ISR_HANDLER GMAC_Q4_Handler
        ISR_HANDLER GMAC_Q5_Handler
#endif
        //
        .section .vectors, "ax"
_vectors_end:

#ifdef __VECTORS_IN_RAM
        //
        // Reserve space with the size of the vector table
        // in the designated RAM section.
        //
        .section .vectors_ram, "ax"
        .balign 512
        .global _vectors_ram

_vectors_ram:
        .space _vectors_end - _vectors, 0
#endif

/*********************************************************************
*
*  Dummy handler to be used for reserved interrupt vectors
*  and weak implementation of interrupts.
*
*/
        .section .init.Dummy_Handler, "ax"
        .thumb_func
        .weak Dummy_Handler
        .balign 2
Dummy_Handler:
        1: b 1b   // Endless loop


/*************************** End of file ****************************/
