/*********************************************************************
*                     SEGGER Microcontroller GmbH                    *
*                        The Embedded Experts                        *
**********************************************************************
*                                                                    *
*       (c) 1995 - 2023 SEGGER Microcontroller GmbH                  *
*                                                                    *
*       Internet: segger.com  Support: support_embos@segger.com      *
*                                                                    *
**********************************************************************
*                                                                    *
*       embOS-Ultra * Real time operating system                     *
*                                                                    *
*       Please note:                                                 *
*                                                                    *
*       Knowledge of this file may under no circumstances            *
*       be used to write a similar product or a real-time            *
*       operating system for in-house use.                           *
*                                                                    *
*       Thank you for your fairness !                                *
*                                                                    *
**********************************************************************
*                                                                    *
*       OS version: V5.18.0.0                                        *
*                                                                    *
**********************************************************************

-------------------------- END-OF-HEADER -----------------------------

File      : LPC55S69_Vectors.s
Purpose   : Exception and interrupt vectors for LPC55S69 devices.

Additional information:
  Preprocessor Definitions
    __NO_EXTERNAL_INTERRUPTS
      If defined,
        the vector table will contain only the internal exceptions
        and interrupts.
    __VECTORS_IN_RAM
      If defined,
        an area of RAM, large enough to store the vector table,
        will be reserved.

    __OPTIMIZATION_SMALL
      If defined,
        all weak definitions of interrupt handlers will share the
        same implementation.
      If not defined,
        all weak definitions of interrupt handlers will be defined
        with their own implementation.
*/
        .syntax unified

/*********************************************************************
*
*       Macros
*
**********************************************************************
*/

//
// Directly place a vector (word) in the vector table
//
.macro VECTOR Name=
        .section .vectors, "ax"
        .code 16
        .word \Name
.endm

//
// Declare an exception handler with a weak definition
//
.macro EXC_HANDLER Name=
        //
        // Insert vector in vector table
        //
        .section .vectors, "ax"
        .word \Name
        //
        // Insert dummy handler in init section
        //
        .section .init.\Name, "ax"
        .thumb_func
        .weak \Name
        .balign 2
\Name:
        1: b 1b   // Endless loop
.endm

//
// Declare an interrupt handler with a weak definition
//
.macro ISR_HANDLER Name=
        //
        // Insert vector in vector table
        //
        .section .vectors, "ax"
        .word \Name
        //
        // Insert dummy handler in init section
        //
#if defined(__OPTIMIZATION_SMALL)
        .section .init, "ax"
        .weak \Name
        .thumb_set \Name,Dummy_Handler
#else
        .section .init.\Name, "ax"
        .thumb_func
        .weak \Name
        .balign 2
\Name:
        1: b 1b   // Endless loop
#endif
.endm

//
// Place a reserved vector in vector table
//
.macro ISR_RESERVED
        .section .vectors, "ax"
        .word 0
.endm

//
// Place a reserved vector in vector table
//
.macro ISR_RESERVED_DUMMY
        .section .vectors, "ax"
        .word Dummy_Handler
.endm

/*********************************************************************
*
*       Externals
*
**********************************************************************
*/
        .extern __stack_end__
        .extern Reset_Handler
        .extern HardFault_Handler

/*********************************************************************
*
*       Global functions
*
**********************************************************************
*/

/*********************************************************************
*
*  Setup of the vector table and weak definition of interrupt handlers
*
*/
        .section .vectors, "ax"
        .code 16
        .balign 512
        .global _vectors
_vectors:
        //
        // Internal exceptions and interrupts
        //
        VECTOR __stack_end__
        VECTOR Reset_Handler
        EXC_HANDLER NMI_Handler
        VECTOR HardFault_Handler
        EXC_HANDLER MemManage_Handler
        EXC_HANDLER BusFault_Handler
        EXC_HANDLER UsageFault_Handler
        EXC_HANDLER SecureFault_Handler
        ISR_RESERVED
        ISR_RESERVED
        ISR_RESERVED
        EXC_HANDLER SVC_Handler
        ISR_RESERVED
        ISR_RESERVED
        EXC_HANDLER PendSV_Handler
        EXC_HANDLER SysTick_Handler
        //
        // External interrupts
        //
#ifndef __NO_EXTERNAL_INTERRUPTS
        ISR_HANDLER WDT_BOD_IRQHandler  /* Windowed watchdog timer, Brownout detect, Flash interrupt */
        ISR_HANDLER DMA0_IRQHandler  /* DMA0 controller */
        ISR_HANDLER GINT0_IRQHandler  /* GPIO group 0 */
        ISR_HANDLER GINT1_IRQHandler  /* GPIO group 1 */
        ISR_HANDLER PIN_INT0_IRQHandler  /* Pin interrupt 0 or pattern match engine slice 0 */
        ISR_HANDLER PIN_INT1_IRQHandler  /* Pin interrupt 1or pattern match engine slice 1 */
        ISR_HANDLER PIN_INT2_IRQHandler  /* Pin interrupt 2 or pattern match engine slice 2 */
        ISR_HANDLER PIN_INT3_IRQHandler  /* Pin interrupt 3 or pattern match engine slice 3 */
        ISR_HANDLER UTICK0_IRQHandler  /* Micro-tick Timer */
        ISR_HANDLER MRT0_IRQHandler  /* Multi-rate timer */
        ISR_HANDLER CTIMER0_IRQHandler  /* Standard counter/timer CTIMER0 */
        ISR_HANDLER CTIMER1_IRQHandler  /* Standard counter/timer CTIMER1 */
        ISR_HANDLER SCT0_IRQHandler  /* SCTimer/PWM */
        ISR_HANDLER CTIMER3_IRQHandler  /* Standard counter/timer CTIMER3 */
        ISR_HANDLER FLEXCOMM0_IRQHandler  /* Flexcomm Interface 0 (USART, SPI, I2C, FLEXCOMM) */
        ISR_HANDLER FLEXCOMM1_IRQHandler  /* Flexcomm Interface 1 (USART, SPI, I2C, FLEXCOMM) */
        ISR_HANDLER FLEXCOMM2_IRQHandler  /* Flexcomm Interface 2 (USART, SPI, I2C, FLEXCOMM) */
        ISR_HANDLER FLEXCOMM3_IRQHandler  /* Flexcomm Interface 3 (USART, SPI, I2C, FLEXCOMM) */
        ISR_HANDLER FLEXCOMM4_IRQHandler  /* Flexcomm Interface 4 (USART, SPI, I2C, FLEXCOMM) */
        ISR_HANDLER FLEXCOMM5_IRQHandler  /* Flexcomm Interface 5 (USART, SPI, I2C,, FLEXCOMM) */
        ISR_HANDLER FLEXCOMM6_IRQHandler  /* Flexcomm Interface 6 (USART, SPI, I2C, I2S,, FLEXCOMM) */
        ISR_HANDLER FLEXCOMM7_IRQHandler  /* Flexcomm Interface 7 (USART, SPI, I2C, I2S,, FLEXCOMM) */
        ISR_HANDLER ADC0_IRQHandler  /* ADC0  */
        ISR_HANDLER Reserved39_IRQHandler  /* Reserved interrupt */
        ISR_HANDLER ACMP_CAPT0_IRQHandler  /* ACMP and CAPT0 interrupts */
        ISR_HANDLER Reserved41_IRQHandler  /* Reserved interrupt */
        ISR_HANDLER Reserved42_IRQHandler  /* Reserved interrupt */
        ISR_HANDLER USB0_NEEDCLK_IRQHandler  /* USB Activity Wake-up Interrupt */
        ISR_HANDLER USB0_IRQHandler  /* USB device */
        ISR_HANDLER RTC_IRQHandler  /* RTC alarm and wake-up interrupts */
        ISR_HANDLER EZH_ARCH_B0_IRQHandler  /* EZH_ARCH_B0 */
        ISR_HANDLER MAILBOX_IRQHandler  /* WAKEUP,Mailbox interrupt (present on selected devices) */
        ISR_HANDLER PIN_INT4_IRQHandler  /* Pin interrupt 4 or pattern match engine slice 4 int */
        ISR_HANDLER PIN_INT5_IRQHandler  /* Pin interrupt 5 or pattern match engine slice 5 int */
        ISR_HANDLER PIN_INT6_IRQHandler  /* Pin interrupt 6 or pattern match engine slice 6 int */
        ISR_HANDLER PIN_INT7_IRQHandler  /* Pin interrupt 7 or pattern match engine slice 7 int */
        ISR_HANDLER CTIMER2_IRQHandler  /* Standard counter/timer CTIMER2 */
        ISR_HANDLER CTIMER4_IRQHandler  /* Standard counter/timer CTIMER4 */
        ISR_HANDLER OS_EVENT_IRQHandler  /* OS_EVENT_TIMER and OS_EVENT_WAKEUP interrupts */
        ISR_HANDLER Reserved55_IRQHandler  /* Reserved interrupt */
        ISR_HANDLER Reserved56_IRQHandler  /* Reserved interrupt */
        ISR_HANDLER Reserved57_IRQHandler  /* Reserved interrupt */
        ISR_HANDLER SDIO_IRQHandler  /* SD/MMC  */
        ISR_HANDLER Reserved59_IRQHandler  /* Reserved interrupt */
        ISR_HANDLER Reserved60_IRQHandler  /* Reserved interrupt */
        ISR_HANDLER Reserved61_IRQHandler  /* Reserved interrupt */
        ISR_HANDLER USB1_UTMI_IRQHandler  /* USB1_UTMI */
        ISR_HANDLER USB1_IRQHandler  /* USB1 interrupt */
        ISR_HANDLER USB1_NEEDCLK_IRQHandler  /* USB1 activity */
        ISR_HANDLER SEC_HYPERVISOR_CALL_IRQHandler  /* SEC_HYPERVISOR_CALL interrupt */
        ISR_HANDLER SEC_GPIO_INT0_IRQ0_IRQHandler  /* SEC_GPIO_INT0_IRQ0 interrupt */
        ISR_HANDLER SEC_GPIO_INT0_IRQ1_IRQHandler  /* SEC_GPIO_INT0_IRQ1 interrupt */
        ISR_HANDLER PLU_IRQHandler  /* PLU interrupt */
        ISR_HANDLER SEC_VIO_IRQHandler  /* SEC_VIO interrupt */
        ISR_HANDLER HASHCRYPT_IRQHandler  /* HASHCRYPT interrupt */
        ISR_HANDLER CASER_IRQHandler  /* CASPER interrupt */
        ISR_HANDLER PUF_IRQHandler  /* PUF interrupt */
        ISR_HANDLER PQ_IRQHandler  /* PQ interrupt */
        ISR_HANDLER DMA1_IRQHandler  /* DMA1 interrupt */
        ISR_HANDLER LSPI_HS_IRQHandler  /* LSPI_HS interrupt */
        ISR_HANDLER PVTVF0_AMBER_IRQHandler  /* PVT interrupts */
        ISR_HANDLER PVTVF0_RED_IRQHandler  /* PVT interrupts */
        ISR_HANDLER PVTVF1_AMBER_IRQHandler  /* PVT interrupts */
        ISR_HANDLER PVTVF1_RED_IRQHandler  /* PVT interrupts */
#endif
        //
        .section .vectors, "ax"
_vectors_end:

#ifdef __VECTORS_IN_RAM
        //
        // Reserve space with the size of the vector table
        // in the designated RAM section.
        //
        .section .vectors_ram, "ax"
        .balign 512
        .global _vectors_ram

_vectors_ram:
        .space _vectors_end - _vectors, 0
#endif

/*********************************************************************
*
*  Dummy handler to be used for reserved interrupt vectors
*  and weak implementation of interrupts.
*
*/
        .section .init.Dummy_Handler, "ax"
        .thumb_func
        .weak Dummy_Handler
        .balign 2
Dummy_Handler:
        1: b 1b   // Endless loop


/*************************** End of file ****************************/
