/*********************************************************************
*                     SEGGER Microcontroller GmbH                    *
*                        The Embedded Experts                        *
**********************************************************************
*                                                                    *
*       (c) 1995 - 2023 SEGGER Microcontroller GmbH                  *
*                                                                    *
*       Internet: segger.com  Support: support_embos@segger.com      *
*                                                                    *
**********************************************************************
*                                                                    *
*       embOS-Ultra * Real time operating system                     *
*                                                                    *
*       Please note:                                                 *
*                                                                    *
*       Knowledge of this file may under no circumstances            *
*       be used to write a similar product or a real-time            *
*       operating system for in-house use.                           *
*                                                                    *
*       Thank you for your fairness !                                *
*                                                                    *
**********************************************************************
*                                                                    *
*       OS version: V5.18.0.0                                        *
*                                                                    *
**********************************************************************

-------------------------- END-OF-HEADER -----------------------------

File      : MSP432P4111_Vectors.s
Purpose   : Exception and interrupt vectors for MSP432P4111 devices.

Additional information:
  Preprocessor Definitions
    __NO_EXTERNAL_INTERRUPTS
      If defined,
        the vector table will contain only the internal exceptions
        and interrupts.
    __VECTORS_IN_RAM
      If defined,
        an area of RAM, large enough to store the vector table, 
        will be reserved.

    __OPTIMIZATION_SMALL
      If defined,
        all weak definitions of interrupt handlers will share the 
        same implementation.
      If not defined,
        all weak definitions of interrupt handlers will be defined
        with their own implementation.
*/
        .syntax unified

/*********************************************************************
*
*       Macros
*
**********************************************************************
*/

//
// Directly place a vector (word) in the vector table
//
.macro VECTOR Name=
        .section .vectors, "ax"
        .code 16
        .word \Name
.endm

//
// Declare an exception handler with a weak definition
//
.macro EXC_HANDLER Name=
        //
        // Insert vector in vector table
        //
        .section .vectors, "ax"
        .word \Name
        //
        // Insert dummy handler in init section
        //
        .section .init.\Name, "ax"
        .thumb_func
        .weak \Name
        .balign 2
\Name:
        1: b 1b   // Endless loop
.endm

//
// Declare an interrupt handler with a weak definition
//
.macro ISR_HANDLER Name=
        //
        // Insert vector in vector table
        //
        .section .vectors, "ax"
        .word \Name
        //
        // Insert dummy handler in init section
        //
#if defined(__OPTIMIZATION_SMALL)
        .section .init, "ax"
        .weak \Name
        .thumb_set \Name,Dummy_Handler
#else
        .section .init.\Name, "ax"
        .thumb_func
        .weak \Name
        .balign 2
\Name:
        1: b 1b   // Endless loop
#endif
.endm

//
// Place a reserved vector in vector table
//
.macro ISR_RESERVED
        .section .vectors, "ax"
        .word 0
.endm

//
// Place a reserved vector in vector table
//
.macro ISR_RESERVED_DUMMY
        .section .vectors, "ax"
        .word Dummy_Handler
.endm

/*********************************************************************
*
*       Externals
*
**********************************************************************
*/
        .extern __stack_end__
        .extern Reset_Handler
        .extern HardFault_Handler

/*********************************************************************
*
*       Global functions
*
**********************************************************************
*/

/*********************************************************************
*
*  Setup of the vector table and weak definition of interrupt handlers
*
*/
        .section .vectors, "ax"
        .code 16
        .balign 512
        .global _vectors
_vectors:
        //
        // Internal exceptions and interrupts
        //
        VECTOR __stack_end__
        VECTOR Reset_Handler
        EXC_HANDLER NMI_Handler
        VECTOR HardFault_Handler
        ISR_RESERVED
        ISR_RESERVED
        ISR_RESERVED
        ISR_RESERVED
        ISR_RESERVED
        ISR_RESERVED
        ISR_RESERVED
        EXC_HANDLER SVC_Handler
        ISR_RESERVED
        ISR_RESERVED
        EXC_HANDLER PendSV_Handler
        EXC_HANDLER SysTick_Handler
        //
        // External interrupts
        //
#ifndef __NO_EXTERNAL_INTERRUPTS
        ISR_HANDLER PSS_IRQHandler
        ISR_HANDLER CS_IRQHandler
        ISR_HANDLER PCM_IRQHandler
        ISR_HANDLER WDT_A_IRQHandler
        ISR_HANDLER FPU_IRQHandler
        ISR_HANDLER FLCTL_A_IRQHandler
        ISR_HANDLER COMP_E0_IRQHandler
        ISR_HANDLER COMP_E1_IRQHandler
        ISR_HANDLER TA0_0_IRQHandler
        ISR_HANDLER TA0_N_IRQHandler
        ISR_HANDLER TA1_0_IRQHandler
        ISR_HANDLER TA1_N_IRQHandler
        ISR_HANDLER TA2_0_IRQHandler
        ISR_HANDLER TA2_N_IRQHandler
        ISR_HANDLER TA3_0_IRQHandler
        ISR_HANDLER TA3_N_IRQHandler
        ISR_HANDLER EUSCIA0_IRQHandler
        ISR_HANDLER EUSCIA1_IRQHandler
        ISR_HANDLER EUSCIA2_IRQHandler
        ISR_HANDLER EUSCIA3_IRQHandler
        ISR_HANDLER EUSCIB0_IRQHandler
        ISR_HANDLER EUSCIB1_IRQHandler
        ISR_HANDLER EUSCIB2_IRQHandler
        ISR_HANDLER EUSCIB3_IRQHandler
        ISR_HANDLER ADC14_IRQHandler
        ISR_HANDLER T32_INT1_IRQHandler
        ISR_HANDLER T32_INT2_IRQHandler
        ISR_HANDLER T32_INTC_IRQHandler
        ISR_HANDLER AES256_IRQHandler
        ISR_HANDLER RTC_C_IRQHandler
        ISR_HANDLER DMA_ERR_IRQHandler
        ISR_HANDLER DMA_INT3_IRQHandler
        ISR_HANDLER DMA_INT2_IRQHandler
        ISR_HANDLER DMA_INT1_IRQHandler
        ISR_HANDLER DMA_INT0_IRQHandler
        ISR_HANDLER PORT1_IRQHandler
        ISR_HANDLER PORT2_IRQHandler
        ISR_HANDLER PORT3_IRQHandler
        ISR_HANDLER PORT4_IRQHandler
        ISR_HANDLER PORT5_IRQHandler
        ISR_HANDLER PORT6_IRQHandler
        ISR_HANDLER LCD_F_IRQHandler
#endif
        //
        .section .vectors, "ax"
_vectors_end:

#ifdef __VECTORS_IN_RAM
        //
        // Reserve space with the size of the vector table
        // in the designated RAM section.
        //
        .section .vectors_ram, "ax"
        .balign 512
        .global _vectors_ram

_vectors_ram:
        .space _vectors_end - _vectors, 0
#endif

/*********************************************************************
*
*  Dummy handler to be used for reserved interrupt vectors
*  and weak implementation of interrupts.
*
*/
        .section .init.Dummy_Handler, "ax"
        .thumb_func
        .weak Dummy_Handler
        .balign 2
Dummy_Handler:
        1: b 1b   // Endless loop


/*************************** End of file ****************************/
